% This work is licensed under a Creative Commons Attribution 4.0
% International License. To view a copy of this license, visit
% https://creativecommons.org/licenses/by/4.0/

% Results from [1, Fig. 7] that are simulated with ADS, i.e., the circuit
% model simulations.
% 
% [1] X. Wang, P.-Y. Qin, L.-Z. Song, R. Jin, and Y. J. Guo, "Tightly
%     coupled Huygens element-based conformal transmitarray for E-band
%     airborne communication systems," IEEE Trans. Antennas Propag.,
%     vol. 71, no. 3, pp. 2467–2475, 2023.
clc; clear variables; close all; drawnow;

same_figure = true;

% 33 points for magnitude.
f_mag = [65.526, 66.316, 66.868, 67.316, 67.789, 68.395, 69.368, ...
         70.184, 70.921, 72.105, 72.921, 74.026, 75.053, 76.000, ...
         76.868, 77.895, 78.711, 79.184, 79.658, 80.342, 80.816, ...
         81.342, 82.105, 82.658, 83.211, 83.816, 84.474, 85.211, ...
         86.211, 87.316, 88.218, 88.989, 89.628];
     
v_mag = [-1.937, -1.653, -1.253, -0.853, -0.453, -0.095, -0.074, ...
         -0.400, -0.726, -1.021, -1.221, -1.211, -1.084, -0.863, ...
         -0.611, -0.358, -0.558, -0.979, -1.347, -1.663, -2.063, ...
         -2.432, -2.232, -1.832, -1.463, -1.084, -0.737, -0.411, ...
         -0.126, -0.063, -0.242, -0.558, -0.958];

% 26 points for phase.
f_pha = [65.526, 66.553, 67.526, 68.474, 69.447, 70.474, 71.605, ...
         72.658, 73.737, 74.842, 75.921, 77.000, 78.000, 78.816, ...
         79.605, 80.079, 80.553, 81.000, 81.868, 82.921, 83.974, ...
         85.053, 86.132, 87.211, 88.298, 89.415];

v_pha = [-243.490, -265.625, -292.969, -321.615, -350.260, -373.698, ...
         -390.625, -403.646, -416.667, -428.385, -441.406, -457.031, ...
         -479.167, -511.719, -558.594, -610.677, -662.760, -712.240, ...
         -752.604, -776.042, -790.365, -805.990, -819.010, -832.031, ...
         -845.052, -859.375];

%% Process HFSS simulation.
th = 0;
ph = 0;

% HFSS gives scattering matrix in TE-TM basis, which needs to be
% transformed to Cartesian basis.
[f_HFSS, v_HFSS] = process_HFSS_simulation('All_S_Tx.tab', th, ph);
[f_CST,  v_CST]  = process_HFSS_simulation('All_S_Tx_CST_v2_post.txt', th, ph);

%% Process Fortran simulation.
[f_For, v_For] = process_Fortran_simulation('frequency_sweep_Tx.dat');

%%
figure; hold on; grid on; grid minor;
if same_figure
    yyaxis left;
end
plot(f_For,  v_For(:,1),  '-',  'color', '#000000', 'linewidth', 2);
plot(f_HFSS, v_HFSS(:,1), '--', 'color', '#0072BD', 'linewidth', 2);
plot(f_CST,  v_CST(:,1),  '-.', 'color', '#D95319', 'linewidth', 2);
plot(f_mag,  v_mag,       ':',  'color', '#EDB120', 'linewidth', 2);
xlabel('$f$ (GHz)', 'interpreter', 'latex');
ylabel('$|\tau_{xx}|$ (dB)', 'interpreter', 'latex');
axis([65, 90, -8, 0]);
if ~same_figure
    legend('This work', 'HFSS', 'CST', 'Circuit model [18]', ...
        'interpreter', 'latex', 'Location', 'SouthWest');
end

if ~same_figure
    figure; hold on; grid on; grid minor;
else
    yyaxis right;
end
% Some phases needs adjustment in y axis. Just add a constant to fit.
plot(f_For,  v_For(:,2),      '-',  'color', '#000000', 'linewidth', 2);
plot(f_HFSS, v_HFSS(:,2)-350, '--', 'color', '#0072BD', 'linewidth', 2);
plot(f_CST,  v_CST(:,2)-180,  '-.', 'color', '#D95319', 'linewidth', 2);
plot(f_pha,  v_pha,           ':',  'color', '#EDB120', 'linewidth', 2);
xlabel('$f$ (GHz)', 'interpreter', 'latex');
ylabel('$\angle \tau_{xx}$ (deg)', 'interpreter', 'latex');
axis([65, 90, -1000, 0]);
if ~same_figure
    legend('This work', 'HFSS', 'CST', 'Circuit model [18]', ...
        'interpreter', 'latex', 'Location', 'SouthWest');
else
    legend('$|\tau_{xx}|$ this work', ...
           '$|\tau_{xx}|$ HFSS', ...
           '$|\tau_{xx}|$ CST', ...
           '$|\tau_{xx}|$ circuit model [18]', ...
           '$\angle \tau_{xx}$ this work', ...
           '$\angle \tau_{xx}$ HFSS', ...
           '$\angle \tau_{xx}$ CST', ...
           '$\angle \tau_{xx}$ circuit model [18]', ...
           'interpreter', 'latex','Location', 'SouthWest');
end

%% Extra functions.
function [f_For, v_For] = process_Fortran_simulation(file)
    
    a = load(file);
    
    f_For = a(:,1);
    
    Txx = complex(a(:,2), a(:,3));
    
    v_For(:,1) = 20*log10(abs(Txx));
    v_For(:,2) = atan2(a(:,3), a(:,2));
    v_For(:,2) = unwrap(v_For(:,2))*180/pi;

    return
end

function [f_HFSS, v_HFSS] = process_HFSS_simulation(file, th, ph)

    fid = fopen(file) ;
    str = repmat('%f', 1, 9);
    data = textscan(fid, str, 'HeaderLines', 2);
    data = cell2mat(data);
    fclose(fid);
    
    f_HFSS = data(:,1);
    
    Tot = numel(f_HFSS);
    v_HFSS = NaN(Tot, 2); % [mag, pha].
    
    S11 = complex(data(:,2), data(:,3));
    S21 = complex(data(:,4), data(:,5));
    S12 = complex(data(:,6), data(:,7));
    S22 = complex(data(:,8), data(:,9));
    
    for n = 1:Tot

        Sm = [S22(n),          S12(n)*cosd(th); ... % Valid if
              S21(n)/cosd(th), S11(n)        ];     % a=x, b=y in HFSS.
          
%       Sm = [S11(n),          S21(n)/cosd(th); ... % Valid if a=y, b=x 
%               S12(n)*cosd(th), S22(n)         ];  % in HFSS.

        M  = [cosd(ph),  sind(ph); ... % If a=x, b=y in HFSS
              sind(ph), -cosd(ph)];
          
%           M  = [-sind(ph), cosd(ph); ... % If a=y, b=x in HFSS.
%                    cosd(ph), sind(ph)];
        
        % M = inv(M).
        Tm = M*Sm*M; %M*Sm/M; % M*Sm*inv(M);
        
        v_HFSS(n, 1) = 20*log10( abs(Tm(1,1)) );
        v_HFSS(n, 2) = atan2(imag(Tm(1,1)), real(Tm(1,1)));
    end
    
    v_HFSS(:,2) = unwrap(v_HFSS(:,2))*180/pi;
end