% This work is licensed under a Creative Commons Attribution 4.0
% International License. To view a copy of this license, visit
% https://creativecommons.org/licenses/by/4.0/

% Optimized cell at 105 GHz. Results from Fortran tool and HFSS at three
% different frequencies.
clc; clear variables; close all; drawnow;

same_figure = true;

color = get(gca,'colororder'); close all;

%% Process HFSS simulation.
th = 0;
ph = 0;
% Converts scattering matrix in TE-TM basis to transmission coefficients in
% Cartesian basis.
[x_HFSS0, y_HFSS0] = process_HFSS_simulation('All_S_Tx_105GHz.tab', th, ph);
[x_HFSS1, y_HFSS1] = process_HFSS_simulation('All_S_Tx_100GHz.tab', th, ph);
[x_HFSS2, y_HFSS2] = process_HFSS_simulation('All_S_Tx_110GHz.tab', th, ph);

%% Process Fortran simulation.
[x_For0, y_For0] = process_Fortran_simulation('fase0_MoM_DRP_MoM_Tx_P4.dat');
[x_For1, y_For1] = process_Fortran_simulation('fase1_MoM_DRP_MoM_Tx_P4.dat');
[x_For2, y_For2] = process_Fortran_simulation('fase2_MoM_DRP_MoM_Tx_P4.dat');

%%
figure; hold on; grid on; grid minor;
if same_figure
    yyaxis left;
end
plot(x_For1, y_For1(:,1), '-', 'color', color(1,:), 'Linewidth', 2);
plot(x_HFSS1, y_HFSS1(:,1), 'o', 'color', color(1,:), 'Linewidth', 2);
plot(x_For0, y_For0(:,1), '-.', 'color', color(2,:), 'Linewidth', 2);
plot(x_HFSS0, y_HFSS0(:,1), '^', 'color', color(2,:), 'Linewidth', 2);
plot(x_For2, y_For2(:,1), ':', 'color', color(3,:), 'Linewidth', 2);
plot(x_HFSS2, y_HFSS2(:,1), 'x', 'color', color(3,:), 'Linewidth', 2);
xlabel('$\beta$', 'interpreter', 'latex');
ylabel('$|\tau_{xx}|$ (dB)', 'interpreter', 'latex');
axis([0.6, 1.15, -8, 0]);
legend('MoM (100 GHz)', 'HFSS (100 GHz)', ...
       'MoM (105 GHz)', 'HFSS (105 GHz)', ...
       'MoM (110 GHz)', 'HFSS (110 GHz)', ...
       'interpreter', 'latex','Location', 'SouthWest', ...
       'NumColumns', 2, 'Orientation', 'horizontal');

if ~same_figure
    figure; hold on; grid on; grid minor;
else
    yyaxis right;
end
plot(x_For1, y_For1(:,2), '-', 'color', color(1,:), 'Linewidth', 2);
plot(x_HFSS1, y_HFSS1(:,2), 'o', 'color', color(1,:), 'Linewidth', 2);
plot(x_For0, y_For0(:,2), '-.', 'color', color(2,:), 'Linewidth', 2);
plot(x_HFSS0, y_HFSS0(:,2), '^', 'color', color(2,:), 'Linewidth', 2);
plot(x_For2, y_For2(:,2), ':', 'color', color(3,:), 'Linewidth', 2);
plot(x_HFSS2, y_HFSS2(:,2), 'x', 'color', color(3,:), 'Linewidth', 2);
xlabel('$\beta$', 'interpreter', 'latex');
ylabel('$\angle \tau_{xx}$ (deg)', 'interpreter', 'latex');
axis([0.6, 1.15, -900, 100]);
xticks(0.6:0.05:1.15);
yticks(-900:125:100);
legend('MoM (100 GHz)', 'HFSS (100 GHz)', ...
       'MoM (105 GHz)', 'HFSS (105 GHz)', ...
       'MoM (110 GHz)', 'HFSS (110 GHz)', ...
       'interpreter', 'latex','Location', 'SouthWest', ...
       'NumColumns', 2, 'Orientation', 'horizontal');

%% Extra functions.
function [x_For, y_For] = process_Fortran_simulation(file)
    
    a = load(file);
    
    x_For = a(:,1);
    
    Txx = complex(a(:,4), a(:,5));
    
    y_For(:,1) = 20*log10(abs(Txx));
    y_For(:,2) = atan2(a(:,5), a(:,4));
    y_For(:,2) = unwrap(y_For(:,2))*180/pi-50;

    return
end

function [f_HFSS, v_HFSS] = process_HFSS_simulation(file, th, ph)

    fid = fopen(file) ;
    str = repmat('%f', 1, 10);
    data = textscan(fid, str, 'HeaderLines', 2);
    data = cell2mat(data);
    fclose(fid);
    
    f_HFSS = data(:,2);
    
    Tot = numel(f_HFSS);
    v_HFSS = NaN(Tot, 2); % [mag, pha].
    
    S11 = complex(data(:,3), data(:,4));
    S21 = complex(data(:,5), data(:,6));
    S12 = complex(data(:,7), data(:,8));
    S22 = complex(data(:,9), data(:,10));
    
    for n = 1:Tot

        Sm = [S22(n),          S12(n)*cosd(th); ... % Valid if
              S21(n)/cosd(th), S11(n)        ];     % a=x, b=y in HFSS.

        M  = [cosd(ph),  sind(ph); ... % If a=x, b=y in HFSS
              sind(ph), -cosd(ph)];
        
        Tm = M*Sm*M; %M*Sm/M; % M*Sm*inv(M);
        
        v_HFSS(n, 1) = 20*log10( abs(Tm(1,1)) );
        v_HFSS(n, 2) = atan2(imag(Tm(1,1)), real(Tm(1,1)));
    end
    % Adjust phase reference. It is the same constant applied to all
    % frequencies.
    v_HFSS(:,2) = unwrap(v_HFSS(:,2))*180/pi-50;
end